@@ -48,6 +48,7 @@ struct _GisAccountPageLocalPrivate
   GtkWidget *subtitle;
   GtkWidget *fullname_entry;
   GtkWidget *username_combo;
+  gboolean   has_custom_username;
   GtkWidget *username_explanation;
   UmPhotoDialog *photo_dialog;
 
@@ -181,12 +182,12 @@ prepopulate_account_page (GisAccountPageLocal *page)
         GError *error = NULL;
         if (!goa_oauth2_based_call_get_access_token_sync (oa2, &token, NULL, NULL, &error))
           {
-            g_warning ("Couldn't get oauth2 token: %s\n", error->message);
+            g_warning ("Couldn't get oauth2 token: %s", error->message);
             g_error_free (error);
           }
         else if (!get_profile_sync (token, &name, &picture, NULL, &error))
           {
-            g_warning ("Couldn't get profile information: %s\n", error->message);
+            g_warning ("Couldn't get profile information: %s", error->message);
             g_error_free (error);
           }
         /* FIXME: collect information from more than one account
@@ -211,7 +212,7 @@ prepopulate_account_page (GisAccountPageLocal *page)
     stream = g_file_read (file, NULL, &error);
     if (!stream)
       {
-        g_warning ("Failed to read picture: %s\n", error->message);
+        g_warning ("Failed to read picture: %s", error->message);
         g_error_free (error);
       }
     else
@@ -298,12 +299,13 @@ fullname_changed (GtkWidget      *w,
 
   gtk_list_store_clear (GTK_LIST_STORE (model));
 
-  if (strlen (name) == 0) {
+  if ((name == NULL || strlen (name) == 0) && !priv->has_custom_username) {
     gtk_entry_set_text (GTK_ENTRY (entry), "");
   }
-  else {
+  else if (name != NULL && strlen (name) != 0) {
     generate_username_choices (name, GTK_LIST_STORE (model));
-    gtk_combo_box_set_active (GTK_COMBO_BOX (priv->username_combo), 0);
+    if (!priv->has_custom_username)
+      gtk_combo_box_set_active (GTK_COMBO_BOX (priv->username_combo), 0);
   }
 
   clear_entry_validation_error (GTK_ENTRY (w));
@@ -319,8 +321,16 @@ username_changed (GtkComboBoxText     *combo,
 {
   GisAccountPageLocalPrivate *priv = gis_account_page_local_get_instance_private (page);
   GtkWidget *entry;
+  const gchar *username;
 
   entry = gtk_bin_get_child (GTK_BIN (combo));
+  username = gtk_entry_get_text (GTK_ENTRY (entry));
+  if (*username == '\0')
+    priv->has_custom_username = FALSE;
+  else if (gtk_widget_has_focus (entry) ||
+           gtk_combo_box_get_active (GTK_COMBO_BOX (priv->username_combo)) > 0)
+    priv->has_custom_username = TRUE;
+
   clear_entry_validation_error (GTK_ENTRY (entry));
 
   priv->valid_username = FALSE;
@@ -403,6 +413,7 @@ gis_account_page_local_constructed (GObject *object)
   gtk_label_set_text (GTK_LABEL (priv->subtitle), _("We need a few details to complete setup."));
   gtk_entry_set_text (GTK_ENTRY (priv->fullname_entry), "");
   gtk_list_store_clear (GTK_LIST_STORE (gtk_combo_box_get_model (GTK_COMBO_BOX (priv->username_combo))));
+  priv->has_custom_username = FALSE;
 
   gtk_image_set_pixel_size (GTK_IMAGE (priv->avatar_image), 96);
   gtk_image_set_from_icon_name (GTK_IMAGE (priv->avatar_image), "avatar-default-symbolic", 1);
@@ -432,7 +443,6 @@ gis_account_page_local_dispose (GObject *object)
   g_clear_object (&priv->goa_client);
   g_clear_object (&priv->avatar_pixbuf);
   g_clear_pointer (&priv->avatar_filename, g_free);
-  g_clear_pointer (&priv->photo_dialog, um_photo_dialog_free);
 
   if (priv->timeout_id != 0) {
     g_source_remove (priv->timeout_id);
