@@ -321,11 +321,10 @@ gis_privacy_page_init (GisPrivacyPage *page)
   gtk_widget_init_template (GTK_WIDGET (page));
 }
 
-void
+GisPage *
 gis_prepare_privacy_page (GisDriver *driver)
 {
-  gis_driver_add_page (driver,
-                       g_object_new (GIS_TYPE_PRIVACY_PAGE,
-                                     "driver", driver,
-                                     NULL));
+  return g_object_new (GIS_TYPE_PRIVACY_PAGE,
+                       "driver", driver,
+                       NULL);
 }
